# -*- encoding: utf-8 -*-
#
# (c) Copyright 2016 Hewlett Packard Enterprise Development LP
# Copyright 2016 Universidade Federal de Campina Grande
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Subscribes the DiscoveryNode.
"""

import abc
import six


@six.add_metaclass(abc.ABCMeta)
class DiscoveryNodeSubscriber(object):
    """
    This class should be implemented by subscribers.
    """
    @abc.abstractmethod
    def available(self, nodes):
        """
        Notify subscribers with events of available nodes.
        """
        raise Exception("NotImplementedException")

    @abc.abstractmethod
    def unavailable(self, nodes):
        """
        Notify subscribers with events of unavailable nodes.
        """
        raise Exception("NotImplementedException")
