# -*- encoding: utf-8 -*-
#
# (c) Copyright 2016 Hewlett Packard Enterprise Development LP
# Copyright 2016 Universidade Federal de Campina Grande
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Default exception class' that is used for the publishers and managers.
"""


class LoginFailException(Exception):
    """ The exception class that is raised when fail login occurs.
    """
    def __init__(self, message=None):
        Exception.__init__(self, message)


class HTTPFailException(Exception):
    """ The exception class that is raised when HTTP error occurs.
    """
    def __init__(self, message=None):
        Exception.__init__(self, message)


class SCMBConnectionFailException(Exception):
    """ The exception class that is raised when the SCMB class cannot connection with RabbitMQ.
    """
    def __init__(self, message=None):
        Exception.__init__(self, message)


class InvalidConfigFileException(Exception):
    """ The exception class that is raised when input config file is not valid.
    """
    def __init__(self, message=None):
        Exception.__init__(self, message)
